import threading
from prometheus_client import start_http_server
from .logger import log
from .file import settings
from .collectors import collectOrchestratorMetrics

def main():
# Loading user inputs via CLI
    fileConfig = settings()
# Setting common used varaibles
    prometheusPort = fileConfig.port()['port']
    orchConfig = fileConfig.orch()
    silverpeakUrl = orchConfig["url"]
    silverpeakKey = orchConfig['key']
    silverpeakSSL = orchConfig['verify_ssl']
    metricsOrchestratorInterval = fileConfig.metricsOrchestrator()['interval']
    debug = fileConfig.debug()['debug']
    Break = fileConfig.Break()['Break']

# Start up the server to expose the metrics.
    start_http_server(prometheusPort)
    log().info(f'starting prometheus exporter on {prometheusPort}')


# Triggering the thread that will collect orchestrator metrics
    if fileConfig.metricsOrchestrator()['collect']:
        log().info(f'starting orchestrator metric collection on {silverpeakUrl}')
        log().info(f'starting orchestrator metric collection every {metricsOrchestratorInterval} seconds ')
        x = threading.Thread(target=collectOrchestratorMetrics, args=(
            silverpeakUrl,
            silverpeakKey,
            silverpeakSSL,
            metricsOrchestratorInterval,
            debug,
            Break,
        ))
        x.start()        
    else:
        log().info(f'skipping orchestrator metric collection')    


if __name__ == '__main__':
    main()


#Importing prometheus library
from prometheus_client import Info,Gauge,Enum

# Orchestrator Metrics
uptime = Info('uptime', 'silverpeak orchestrator uptime',['orchName'])
release = Info('release', 'silverpeak orchestrator release version',['orchName'])
platform = Info('platform', 'silverpeak orchestrator platform',['orchName'])
numActiveUsers = Gauge('numActiveUsers', 'number of active users logged in',['orchName'])
activeAlarms = Gauge('activeAlarms', 'summary of active alarms for Orchestrator and all appliances',['orchName','severity'])
totalAppliances = Gauge('totalAppliances', 'number of total appliances in your orchestrator',['orchName'])
licenseNumMini  = Gauge('licenseNumMini', 'number of mini licenses',['orchName'])
licenseNumBase  = Gauge('licenseNumBase', 'number of base licenses',['orchName'])
licenseNumBoost  = Gauge('licenseNumBoost', 'number of boost licenses',['orchName'])
licenseNumPlus  = Gauge('licenseNumPlus', 'number of plus licenses',['orchName'])
licenseNumTier20M = Gauge('licenseNumTier20M', 'number of 20M licenses',['orchName'])
licenseNumTier50M = Gauge('licenseNumTier50M', 'number of 50M licenses',['orchName'])
licenseNumTier100M = Gauge('licenseNumTier100M', 'number of 100M licenses',['orchName'])
licenseNumTier200M = Gauge('licenseNumTier200M', 'number of 200M licenses',['orchName'])
licenseNumTier500M = Gauge('licenseNumTier500M', 'number of 500M licenses',['orchName'])
licenseNumTier1G = Gauge('licenseNumTier1G', 'number of 1G licenses',['orchName'])
licenseNumTier2G = Gauge('licenseNumTier2G', 'number of 2G licenses',['orchName'])
licenseNumTierUM = Gauge('licenseNumTierUM', 'number of Unlimited licenses',['orchName'])
totalOverlays = Gauge('totalOverlays', 'number of overlays configured',['orchName'])
totalRegions = Gauge('totalRegions', 'number of regions configured',['orchName'])
totalSaasApps = Gauge('totalSaasApps', 'list of unique SaaS applications in the internet services database',['orchName'])
totalAppDefinition = Gauge('totalAppDefinition', 'count for application definition data for Address Map from Cloud Portal',['orchName'])
totalOrchSaasApps = Gauge('totalOrchSaasApps', 'count of internet services defined on Orchestrator',['orchName'])
orchPortalStatus = Enum('orchPortalStatus', 'current connectivity status between Orchestrator and Cloud Portal',['orchName'], states=['unable to connect', 'connected', 'connecting'])
cloudPortalServices = Gauge('cloudPortalServices', 'silverpeak cloud portal service and status',['orchName','portalService','status'])


import os
from .logger import log

def createDebugFolder(debug:bool=False):
    if debug == True:
        log().info("starting program in debug mode")
        pathDir = os.path.dirname(os.path.realpath(__file__))
        listOfFolders = os.listdir(pathDir)
        debugFolder = os.path.join(pathDir, "debug")
        debugFile = os.path.join(debugFolder,'debug.log')
        if 'debug' in listOfFolders:
            log().info(f"creating new debug.txt file on {debugFolder}")
            file = open(debugFile, 'w+')
            file.close()
        else:
            try:
                log().info(f"creating debug folder to store files at {debugFolder}")
                os.mkdir(debugFolder)
                file = open(debugFile, 'w+')
                file.close()
            except Exception as error:
                log().error(error)
    else:
        pass


def confirmReturn(func , dictionary:dict, debug:bool):
    if debug == True:
        try:
            if dictionary['status_code'] != 200:
                log().error({func : dictionary})
        except:
            pass
    else:
        pass

def errorHandler(func):
    def wrapper(*args):
        try:
            out = func(*args)
        except Exception as error:
            log().error({func : error})
        return {func : out}
    return wrapper



import inspect
from pyedgeconnect import Orchestrator
from prometheus_client import Info,Gauge
from time import sleep as wait
from .logger import log,logToFile
from .utls import confirmReturn,errorHandler
from .metrics import *


class collectOrchestratorMetrics():
    def __init__(self,url,key,verify_ssl,interval,debug, Break):
        self.url = url
        self.key = key
        self.verify_ssl = verify_ssl
        self.interval = interval
        self.debug = debug
        self.Break = Break
        self.orch = Orchestrator(url=self.url, verify_ssl=self.verify_ssl, api_key=self.key , log_console=self.debug)
        self.hostname = self.orch.get_orchestrator_hostname()['gms_hostname']
        while True:
            # get a list of all the methods in this class
            # loops over the list starting at index 1 to bypass __init__
            # calls the method
            methodList = inspect.getmembers(self, predicate=inspect.ismethod)
            for m in range(1, len(methodList)):
                i = methodList[m][1]()

                logToFile().debug(message=dict({methodList[m][1].__name__ : i}), debug=self.debug) 
                confirmReturn(func=methodList[m][1].__name__ ,dictionary=i, debug=self.debug) 

            if self.Break == False:
                wait(self.interval)
            else:
                break

    @errorHandler
    def _getOrchServerBrief(self):
       orch_return = self.orch.get_orchestrator_server_info()   
       uptime.labels(orchName=self.hostname).info({'uptime' : orch_return['uptime']}) #Setting Metric
       release.labels(orchName=self.hostname).info({'release' : orch_return['release']}) #Setting Metric
       platform.labels(orchName=self.hostname).info({'platform' : orch_return['platform']}) #Setting Metric
       numActiveUsers.labels(orchName=self.hostname).set(orch_return['numActiveUsers']) #Setting Metric
       return orch_return

    @errorHandler
    def _getAlarmCountOrchestratoAndAppliances(self):
        orch_return = self.orch.get_alarm_count_orchestrator_and_appliances() 
        for key,value in orch_return.items():
            activeAlarms.labels(orchName=self.hostname,severity=key).set(value) #Setting Metric
        return orch_return                

    @errorHandler
    def _getTotalAppliances(self):
        orch_return = self.orch.get_appliances() 
        totalAppliances.labels(orchName=self.hostname).set(len(orch_return)) #Setting Metric
        return orch_return


    @errorHandler
    def _getLicenseState(self):
        orch_return = self.orch.get_portal_licensed_summary() 
        licenseState = orch_return['licenseState']['fx']
        
        licenseNumMini.labels(orchName=self.hostname).set(licenseState['numMini']) #Setting Metric
        licenseNumBase.labels(orchName=self.hostname).set(licenseState['numBase']) #Setting Metric
        licenseNumBoost.labels(orchName=self.hostname).set(licenseState['numBoost']) #Setting Metric
        licenseNumPlus.labels(orchName=self.hostname).set(licenseState['numPlus']) #Setting Metric
        numTier = {
            "20000" : licenseNumTier20M,
            "50000" : licenseNumTier50M,
            "100000" : licenseNumTier100M,
            "200000" : licenseNumTier200M,
            "500000" : licenseNumTier500M,
            "1000000" : licenseNumTier1G,
            "2000000" : licenseNumTier2G,
            "1000000000" : licenseNumTierUM,
        }
        for key,value in numTier.items():
            if key in licenseState['numTier']:
                value.labels(orchName=self.hostname).set(licenseState['numTier'][key]) #Setting Metric
            else:
                value.labels(orchName=self.hostname).set(0) #Setting Metric
        return orch_return

    @errorHandler
    def _getTotalOverlays(self):
        orch_return = self.orch.get_all_overlays_config() 
        totalOverlays.labels(orchName=self.hostname).set(len(orch_return)) #Setting Metric
        return orch_return

    @errorHandler
    def _getTotalRegions(self):
        orch_return = self.orch.get_all_regions() 
        totalRegions.labels(orchName=self.hostname).set(len(orch_return)) #Setting Metric
        return orch_return

    @errorHandler
    def _getTotalSaasApps(self):
        orch_return = self.orch.get_all_saas_apps() 
        totalSaasApps.labels(orchName=self.hostname).set(len(orch_return)) #Setting Metric
        return orch_return

    @errorHandler
    def _getTotalAppDefinition(self):
        orch_return = self.orch.get_app_definition_total() 
        totalAppDefinition.labels(orchName=self.hostname).set(orch_return) #Setting Metric
        return orch_return

    @errorHandler
    def _getTotalOrchSaasApps(self):
        orch_return = self.orch.get_count_of_saas_apps() 
        totalOrchSaasApps.labels(orchName=self.hostname).set(orch_return['count']) #Setting Metric
        return orch_return

    @errorHandler
    def _getOrchPortalStatus(self):
        orch_return = self.orch.get_orchestrator_to_cloud_portal_status()
        mapping = {
            0 : 'unable to connect',
            1 : 'connected',
            2 : 'connecting'
        }
        orchPortalStatus.labels(orchName=self.hostname).state(mapping[orch_return['portal']]) #Setting Metric
        return orch_return
            
    @errorHandler
    def _getCloudPortalServices(self):
        orch_return = self.orch.get_portal_services_status() 
        for key,value in orch_return.items():
            cloudPortalServices.labels(orchName=self.hostname,portalService=key, status='ok').set(orch_return[key]['state']['ok'])
            cloudPortalServices.labels(orchName=self.hostname,portalService=key, status='fail').set(orch_return[key]['state']['fail'])

        return orch_return

    @errorHandler
    def _test(self):
        orch_return = self.orch.get_portal_services_status() 
        return orch_return
